#include <gccore.h>
#include "../../emu/h/types.h"
#include "../../nes/h/nesGeneral.h"
#include "../../nes/h/cpu.h"

#include "input.h"

char padOppositeLock = 1;				//cant on a real pad anyways, but it can mapped to buttons!
int padNum;
PADStatus gcPad[4];
PadStatusHit	keys[4][8];

char buttonHit(char nesPadNum, char gcPadNum, char buttonNum)
{
	return
		gcPad[padNum].button & keys[padNum][buttonNum].button ||
		(keys[nesPadNum][buttonNum].stickX_Left		? gcPad[gcPadNum].stickX < keys[nesPadNum][buttonNum].stickX_Left		: 0) ||
		(keys[nesPadNum][buttonNum].stickX_Right	? gcPad[gcPadNum].stickX > keys[nesPadNum][buttonNum].stickX_Right		: 0) ||
		(keys[nesPadNum][buttonNum].stickY_Up		? gcPad[gcPadNum].stickY < keys[nesPadNum][buttonNum].stickY_Up		: 0) ||
		(keys[nesPadNum][buttonNum].stickY_Down		? gcPad[gcPadNum].stickY > keys[nesPadNum][buttonNum].stickY_Down		: 0) ||

		(keys[nesPadNum][buttonNum].substickX_Left	? gcPad[gcPadNum].substickX < keys[nesPadNum][buttonNum].substickX_Left	: 0) ||
		(keys[nesPadNum][buttonNum].substickX_Right	? gcPad[gcPadNum].substickX > keys[nesPadNum][buttonNum].substickX_Right: 0) ||
		(keys[nesPadNum][buttonNum].substickY_Up	? gcPad[gcPadNum].substickY < keys[nesPadNum][buttonNum].substickY_Up	: 0) ||
		(keys[nesPadNum][buttonNum].substickY_Down	? gcPad[gcPadNum].substickY > keys[nesPadNum][buttonNum].substickY_Down	: 0) ||

		(keys[nesPadNum][buttonNum].triggerL		? gcPad[gcPadNum].triggerL > keys[nesPadNum][buttonNum].triggerL	: 0) ||
		(keys[nesPadNum][buttonNum].triggerR		? gcPad[gcPadNum].triggerR > keys[nesPadNum][buttonNum].triggerR	: 0)
		;
}

void BuildKeys()		//The pad may be wrong, lets test & find out
{
	int i;
	PAD_Read(gcPad);

	pad[0] = pad[1] = 0;
	for (i=0; i<8; i++)
	{
		pad[0] <<=1;
		pad[0]  |= !buttonHit(0, 0, i);
		pad[1] <<=1;
		pad[1]  |= !buttonHit(1, 1, i);
	}
	if (padOppositeLock)
	{
		for (char n=0; n<2; n++)
		{
			if ((~pad[n] & 0x80) && (~pad[n] & 0x40))
				pad[n] |= 0x80;

			if ((~pad[n] & 0x20) && (~pad[n] & 0x10))
				pad[n] |= 0x20;
		}
	}
}

void configurePadButton( char padNumber,
		short nes_A,	short nes_B,	short nes_Select,	short nes_Start,
		short nes_Up,	short nes_Down,	short nes_Left,		short nes_Right)
{
	keys[padNumber][nesA].button		= nes_A;
	keys[padNumber][nesB].button		= nes_B;
	keys[padNumber][nesSelect].button	= nes_Select;
	keys[padNumber][nesStart].button	= nes_Start;

	keys[padNumber][nesUp].button		= nes_Up;
	keys[padNumber][nesDown].button		= nes_Down;
	keys[padNumber][nesLeft].button		= nes_Left;
	keys[padNumber][nesRight].button	= nes_Right;
}


void configurePadStick( char padNumber, char nesButton,
	   	s8 stickX_Left,
		s8 stickX_Right,
		s8 stickY_Up,
		s8 stickY_Down,

		s8 substickX_Left,
		s8 substickX_Right,
		s8 substickY_Up,
		s8 substickY_Down,

		u8 triggerL,
		u8 triggerR)
{
	keys[padNumber][nesButton].stickX_Left		= stickX_Left;
	keys[padNumber][nesButton].stickX_Right		= stickX_Right;
	keys[padNumber][nesButton].stickY_Up		= stickY_Up;
	keys[padNumber][nesButton].stickY_Down		= stickY_Down;
	keys[padNumber][nesButton].substickX_Left	= substickX_Left;
	keys[padNumber][nesButton].substickX_Right	= substickX_Right;
	keys[padNumber][nesButton].substickY_Up		= substickY_Up;
	keys[padNumber][nesButton].substickY_Down	= substickY_Down;
	keys[padNumber][nesButton].triggerL			= triggerL;
	keys[padNumber][nesButton].triggerR			= triggerR;
}

void loadDefaultKeys()
{
	int i;
	for (i = 0; i<1; i++)
	{
		configurePadButton(0, 
			PAD_BUTTON_A,
			PAD_BUTTON_B,
			PAD_BUTTON_X | PAD_TRIGGER_Z,
			PAD_BUTTON_Y | PAD_BUTTON_START,
			PAD_BUTTON_UP,
			PAD_BUTTON_DOWN,
			PAD_BUTTON_LEFT,
			PAD_BUTTON_RIGHT);

		configurePadStick( i, nesA,		 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesB,		 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesSelect, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesStart,	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

		configurePadStick( i, nesUp,	 0, 0, 32, 0, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesDown,	 0, 0, 0, 32, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesUp,	 0, 0, 32, 0, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesDown,	 0, 0, 0, 32, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesLeft,	 32, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		configurePadStick( i, nesRight,	 0, 32, 0, 0, 0, 0, 0, 0, 0, 0);
	}
}
